// -*- C++ -*-
/*!
 * @file RtcFactory.h
 * @brief RTComponent factory class
 * @date $Date: 2005/05/16 06:06:30 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RtcFactory.h,v 1.2 2005/05/16 06:06:30 n-ando Exp $
 *
 */

/*
 * $Log: RtcFactory.h,v $
 * Revision 1.2  2005/05/16 06:06:30  n-ando
 * - RtcFactoryBase, RtcFactoryCXX classes were DLL exported for Windows port.
 *
 * Revision 1.1.1.1  2005/05/12 09:06:18  n-ando
 * Public release.
 *
 *
 */

#ifndef RtcFactory_h
#define RtcFactory_h

#include "rtm/RtcModuleProfile.h"


namespace RTM 
{
  class RtcBase;
  class RtcManager;
  /*!
   * @if jp
   *
   * @class RtcFactoryBase
   *
   * @brief RtcFactoryBase 쥯饹
   * 
   * ݡͥȥեȥδ쥯饹
   *
   * @else
   *
   * @class RtcFactoryBase
   *
   * @brief RtcFactoryBase base class
   *
   * RTComponent factory base class.
   *
   * @endif
   */
  class EXPORTS RtcFactoryBase
  {
  public:
	/*!
	 * @if jp
	 *
	 * @brief RtcFactoryBase 饹󥹥ȥ饯
	 *
	 * RtcFactoryBase 饹Υ󥹥ȥ饯
	 *
	 * @param profile ݡͥȤΥץե
	 *
	 * @else
	 *
	 * @brief RtcFactoryBase class constructor.
	 *
	 * RtcFactoryBase class constructor.
	 *
	 * @param profile component profile
	 *
	 * @endif
	 */
    RtcFactoryBase(RtcModuleProfile profile)
      : m_Profile(profile), m_Number(-1) {};
	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥȤ
	 *
	 * Python Ǽ줿 RTComponent Υ󥹥󥹤롣
	 * 貾۴ؿ
	 *
	 * @param mgr RtcManager ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Create component
	 *
	 * Create component implemented in Python.
	 * Pure virtual method.
	 *
	 * @param mgr pointer to RtcManager
	 *
	 * @endif
	 */
    virtual RtcBase* create(RtcManager* mgr) = 0;
	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥȤ˴
	 *
	 * RTComponent Υ󥹥󥹤˴롣
	 * 貾۴ؿ
	 *
	 * @param comp RtcBase ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Destroy component
	 *
	 * Destroy component instance
	 * Pure virtual method.
	 *
	 * @param comp pointer to RtcBase
	 *
	 * @endif
	 */
    virtual void destroy(RtcBase* comp) = 0;
	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥȥץեμ
	 *
	 * ݡͥȤΥץե
	 *
	 * @else
	 *
	 * @brief Get component profile
	 *
	 * Get component profile.
	 *
	 * @endif
	 */
    virtual RtcModuleProfile& profile() {return m_Profile;};
	
	/*!
	 * @if jp
	 *
	 * @brief ߤΥ󥹥󥹿
	 *
	 * ݡͥȤθߤΥ󥹥󥹿롣
	 *
	 * @else
	 *
	 * @brief Get number of component instances
	 *
	 * Get number of current component instances.
	 *
	 * @endif
	 */
    virtual int& number() {return m_Number;};
	
  protected:
	/*!
	 * @if jp
	 * @brief ݡͥȤΥץե
	 * @else
	 * @brief Component profile
	 * @endif
	 */
    RtcModuleProfile m_Profile;
	/*!
	 * @if jp
	 * @brief ߤΥ󥹥󥹿
	 * @else
	 * @brief Number of current component instances.
	 * @endif
	 */
	int m_Number;
  };
  
  
  
  /*!
   * @if jp
   *
   * @class RtcFactoryCXX
   *
   * @brief RtcFactoryCXX 饹
   * 
   * C++ѥݡͥȥեȥꥯ饹
   *
   * @else
   *
   * @class RtcFactoryCXX
   *
   * @brief RtcFactoryCXX class
   *
   * RTComponent factory class for C++.
   *
   * @endif
   */
  class EXPORTS RtcFactoryCXX
    : public RtcFactoryBase
  {
  public:
	/*!
	 * @if jp
	 *
	 * @brief RtcFactoryCXX 饹󥹥ȥ饯
	 *
	 * RtcFactoryCXX 饹Υ󥹥ȥ饯
	 * ץե롢ؿؤΥݥ󥿡˴ؿؤΥݥ󥿤˼ꡢ
	 * C++ Ǽ줿ݡͥȤΥեȥꥯ饹롣
	 *
	 * @param profile ݡͥȤΥץե
	 * @param new_func ݡͥȤؿؤΥݥ
	 * @param delete_func ݡͥȤ˴ؿؤΥݥ
	 *
	 * @else
	 *
	 * @brief RtcFactoryCXX class constructor.
	 *
	 * RtcFactoryCXX class constructor.
	 * Create component factory class with three arguments:
	 * component profile, function pointer to object create function and
	 * object delete function.
	 *
	 * @param profile Component profile
	 * @param new_func Pointer to component create function
	 * @param delete_func Pointer to component delete function
	 *
	 * @endif
	 */
    RtcFactoryCXX(RtcModuleProfile profile,
				  RtcNewFunc new_func,
				  RtcDeleteFunc delete_func);
	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥȤ
	 *
	 * Python Ǽ줿 RTComponent Υ󥹥󥹤롣
	 *
	 * @param mgr RtcManager ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Create component
	 *
	 * Create component implemented in Python.
	 *
	 * @param mgr pointer to RtcManager
	 *
	 * @endif
	 */
    virtual RtcBase* create(RtcManager* mgr);
	
	/*!
	 * @if jp
	 *
	 * @brief ݡͥȤ˴
	 *
	 * RTComponent Υ󥹥󥹤˴롣
	 *
	 * @param comp RtcBase ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Destroy component
	 *
	 * Destroy component instance
	 *
	 * @param comp pointer to RtcBase
	 *
	 * @endif
	 */
    virtual void destroy(RtcBase* comp);
	
  protected:
	/*!
	 * @if jp
	 * @brief ݡͥȥ֥ؿؤΥݥ
	 * @else
	 * @brief The pointer to component object create function
	 * @endif
	 */
    RtcNewFunc m_New;
	
	/*!
	 * @if jp
	 * @brief ݡͥȥ֥˴ؿؤΥݥ
	 * @else
	 * @brief The pointer to component object delete function
	 * @endif
	 */
    RtcDeleteFunc m_Delete;
  };
};


#endif // RtcFactory_h
