// -*- C++ -*-
/*!
 * @file RtcInPort.h
 * @brief InPort template class
 * @date $Date: 2005/05/16 06:12:15 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RtcInPort.h,v 1.2 2005/05/16 06:12:15 n-ando Exp $
 *
 */

/*
 * $Log: RtcInPort.h,v $
 * Revision 1.2  2005/05/16 06:12:15  n-ando
 * - Time variables of "m_Value" were initialized in constructors.
 *
 * Revision 1.1.1.1  2005/05/12 09:06:18  n-ando
 * Public release.
 *
 *
 */

#ifndef RtcInPort_h
#define RtcInPort_h

#include "rtm/RtcInPortBase.h"
#include "rtm/idl/RTCDataTypeSkel.h"
#include <iostream>

namespace RTM
{
  /*!
   * @if jp
   *
   * @class InPortAny
   *
   * @brief InPort ƥץ졼ȥ饹
   * 
   * InPort μǤ InPortAny<T> Υƥץ졼ȥ饹
   * <T> RTCDataType.idl ˤƤ뷿ǡФȤ
   * Time  tm ,  T data Ĺ¤ΤǤʤƤϤʤʤ
   * InPort ˥󥰥Хåե줿ǡ缡
   * Υ󥰥Хåե˳Ǽ롣󥰥ХåեΥϥǥեȤ64
   * ʤäƤ뤬󥹥ȥ饯ˤꥵꤹ뤳ȤǤ롣
   * ǡϥե饰ˤä̤ɡɾ֤졢isNew(), getNewDataLen()
   * getNewList(), getNewListReverse() Υ᥽åɤˤϥɥ󥰤뤳Ȥ
   * Ǥ롣
   *
   * @else
   *
   * @class InPortAny
   *
   * @brief OutPort template class
   *
   * This class template provides interfaces to input port.
   * Component developer can define input value, which act as input
   * port from other components, using this template.
   * This is class template. This class have to be incarnated class as port
   * value types. This value types are previously define RtComponent IDL.
   * ex. type T: TimedFload, TimedLong etc... 
   *
   * @endif
   */
  template <class T> class InPortAny
	: public InPortBase
  {
  public:
	/*!
	 * @if jp
	 *
	 * @brief InPortAny 饹󥹥ȥ饯
	 *
	 * InPortAny<T> 饹Υ󥹥ȥ饯
	 * ѥ᡼ȤͿ T ѿ˥Хɤ롣
	 *
	 * @param name InPort ̾InPortBase:name() ˤ껲Ȥ롣
	 * @param value  InPort ˥Хɤ T ѿ
	 * @param bufsize InPort Υ󥰥ХåեΥХåեĹ
	 *
	 * @else
	 *
	 * @brief A constructor.
	 *
	 * Setting channel name and registering channel value.
	 *
	 * @param name A name of the InPort. This name is referred by
	 *             InPortBase::name().
	 * @param value A channel value related with the channel.
	 * @param bufsize Buffer length of internal ring buffer of InPort 
	 *
	 * @endif
	 */
	InPortAny(const char* name, T& value, int bufsize=64)
	  : m_Buffer(bufsize), m_Value(value)
	{
	  // Set PortProfile
	  m_Profile.name = CORBA::string_dup(name);
	  CORBA::Any var;
	  var <<= m_Value;
	  m_Profile.port_type = var.type();

	  // Init time of data variable
	  m_Value.tm.sec = 0;
	  m_Value.tm.nsec = 0;
	  
	  // Initializing ring buffer
	  for (int i = 0 ; i <= m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(value);
		}
	};

	/*!
	 * @if jp
	 *
	 * @brief InPortAny 饹󥹥ȥ饯
	 *
	 * InPortAny<T> 饹Υ󥹥ȥ饯
	 * 
	 *
	 * @param name InPort ̾InPortBase:name() ˤ껲Ȥ롣
	 * @param bufsize InPort Υ󥰥ХåեΥХåեĹ
	 *
	 * @else
	 *
	 * @brief A constructor.
	 *
	 * Setting channel name and registering channel value.
	 *
	 * @param name A name of the InPort. This name is referred by
	 *             InPortBase::name().
	 * @param bufsize Buffer length of internal ring buffer of InPort 
	 *
	 * @endif
	 */
	InPortAny(const char* name, int bufsize=64)
	  : m_Buffer(bufsize), m_Value(m_Dummy)
	{
	  // Set PortProfile
	  m_Profile.name = CORBA::string_dup(name);
	  CORBA::Any var;
	  var <<= m_Value;
	  m_Profile.port_type = var.type();

	  // Init time of data variable
	  m_Value.tm.sec = 0;
	  m_Value.tm.nsec = 0;

	  // Initializing ring buffer
	  for (int i = 0 ; i <= m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(m_Value);
		}
	};
	
	
	/*!
	 * @if jp
	 *
	 * @brief InPortAny 饹ǥȥ饯
	 *
	 * InPortAny<T> 饹Υǥȥ饯
	 *
	 * @else
	 *
	 * @brief A destructor
	 *
	 * @endif
	 */
	virtual ~InPortAny(){};

	/*!
	 * @if jp
	 *
	 * @brief [CORBA interface] InPort ͤͿ
	 *
	 * InPort ͤ put 롣
	 *
	 * @else
	 *
	 * @brief [CORBA interface] Put data on InPort
	 *
	 * @endif
	 */
	virtual void put(const CORBA::Any& value)
	  throw (RTM::PortBase::Disconnected, CORBA::SystemException)
	{
	  T* tmp;

	  if (value >>= tmp)
		m_Buffer.put(*tmp);
	  return;
	}
	
	/*!
	 * @if jp
	 *
	 * @brief InPort Υ󥰥Хåեͤ
	 *
	 * InPort Υ󥰥Хåեͤ롣
	 *
	 * @else
	 *
	 * @brief Initialize ring buffer value of InPort
	 *
	 * @endif
	 */
	virtual void initBuffer(T& value)
	{
	  for (int i = 0 ; i <= m_Buffer.buff_length(); i++)
		{
		  m_Buffer.put(value);
		}
	}


	/*!
	 * @if jp
	 *
	 * @brief Хɤ줿 T ѿ InPort Хåեκǿͤɤ߹
	 *
	 * Хɤ줿 T Υǡ InPort κǿͤɤ߹ࡣ
	 * 󥹥ȥ饯 T ѿ InPort ХɤƤʤФʤʤ
	 * Υ᥽åɤϥݥ⡼եå˻ѤȤƤ뤿ᡢ
	 * ˰¸ʤͤȤʤäƤ롣
	 *
	 * @else
	 *
	 * @brief Read into bound T-type data from current InPort
	 *
	 * @endif
	 */
	virtual void read_pm()
	{
	  m_Value = m_Buffer.get_new();
	  return;
	};
	
	/*!
	 * @if jp
	 *
	 * @brief Хɤ줿 T ѿ InPort Хåեκǿͤɤ߹
	 *
	 * Хɤ줿 T Υǡ InPort κǿͤɤ߹ࡣ
	 * 󥹥ȥ饯 T ѿ InPort ХɤƤʤФʤʤ
	 *
	 * @else
	 *
	 * @brief Read into bound T-type data from current InPort
	 *
	 * @endif
	 */
	virtual T read()
	{
	  m_Value = m_Buffer.get_new();
	  return m_Value;
	};
	
	/*!
	 * @if jp
	 *
	 * @brief T Υǡ InPort κǿͥǡɤ߹
	 *
	 * @param rhs InPort Хåեͤɤ߹ T ѿ
	 *
	 * @else
	 *
	 * @brief Read from InPort buffer to type-T variable
	 *
	 * @param rhs type-T variable which is wrote from InPort buffer
	 *
	 * @endif
	 */
	virtual bool operator>>(T& rhs)
	{
	  rhs = m_Buffer.get_new();
	  return true;
	};

	/*!
	 * @if jp
	 *
	 * @brief ǿǡ̤ɤοǡɤĴ٤
	 *
	 * @else
	 *
	 * @brief Check the newest data is new data to be read.
	 *
	 * @endif
	 */
	virtual bool isNew()
	{
	  return m_Buffer.is_new();
	}


	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡ
	 *
	 * @else
	 *
	 * @brief Get number of new data to be read.
	 *
	 * @endif
	 */
	virtual int getNewDataLen()
	{
	  return m_Buffer.new_data_len();
	}


	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡ
	 *
	 * @else
	 *
	 * @brief Get new data to be read.
	 *
	 * @endif
	 */
	virtual std::vector<T> getNewList()
	{
	  return m_Buffer.get_new_list();
	}
	

	/*!
	 * @if jp
	 *
	 * @brief ̤ɤοǡս(->)Ǽ
	 *
	 * @else
	 *
	 * @brief Get new data to be read.
	 *
	 * @endif
	 */
	virtual std::vector<T> getNewListReverse()
	{
	  return m_Buffer.get_new_rlist();
	}
	

  private:
	/*!
	 * @if jp
	 * @brief Хɤ T ѿؤλ
	 * @else
	 * @brief reference to type-T value bound this OutPort
	 * @endif
	 */
	T& m_Value;

	/*!
	 * @if jp
	 * @brief T Υߡѿ
	 * @else
	 * @brief Dummy variable of type-T
	 * @endif
	 */
	T  m_Dummy;

	/*!
	 * @if jp
	 * @brief T ΰѿ
	 * @else
	 * @brief Tempolary variable of type-T
	 * @endif
	 */
	T* m_Tmp;

	/*!
	 * @if jp
	 * @brief Хåե
	 * @else
	 * @brief Buffer
	 * @endif
	 */
	RingBuffer<T> m_Buffer;

  };
  
}; // End of namesepace RTM

#endif // RtcInPort_h
