// -*- C++ -*-
/*!
 * @file RtcNaming.h
 * @brief naming Service helper class
 * @date $Date: 2005/05/16 06:29:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RtcNaming.h,v 1.2 2005/05/16 06:29:05 n-ando Exp $
 *
 */

/*
 * $Log: RtcNaming.h,v $
 * Revision 1.2  2005/05/16 06:29:05  n-ando
 * - RtcNaming class was DLL exported for Windows port.
 *
 * Revision 1.1.1.1  2005/05/12 09:06:18  n-ando
 * Public release.
 *
 *
 */


#ifndef RtcNaming_h
#define RtcNaming_h

// CORBA header include
#include "rtm/idl/RTMBaseStub.h"
#include "rtm/idl/RTMBaseSkel.h"
#include "rtm/RtcBase.h"

// STL includes
#include <map>
#include <string>
#include <vector>

#define HOST_CTXT_LEVEL 1
#define MANAGER_CTXT_LEVEL 2
#define MANAGER_LEVEL 3
#define CATEGORY_CTXT_LEVEL 3
#define MODULE_CTXT_LEVEL 4
#define COMPONENT_LEVEL 5

namespace RTM
{
  using namespace std;
  
  typedef std::vector<CosNaming::NamingContextExt_var> ContextList;
  typedef std::vector<CORBA::Object_ptr> ObjectList;


  /*!
   * @if jp
   *
   * @class RtcNaming
   *
   * @brief CORBA Naming Service إѡ饹
   *
   * @else
   *
   * @class RtcNaming
   *
   * @brief CORBA Naming Service access helper class
   *
   * @endif
   */
  class EXPORTS RtcNaming
  {
  public:
	/*!
	 * @if jp
	 * @brief RtcNaming 饹󥹥ȥ饯
	 *
	 * @else
	 *
	 * @brief A constructor of RtcNaming class.
	 *
	 * @endif
	 */
    RtcNaming() : m_bindListNum(100) {};
	
	/*!
	 * @if jp
	 * @brief RtcNaming 饹󥹥ȥ饯
	 *
	 * @else
	 *
	 * @brief A constructor of RtcNaming class.
	 *
	 * @endif
	 */
    RtcNaming(CORBA::ORB_ptr orb) : m_bindListNum(100) {initNaming(orb);};
	
	/*!
	 * @if jp
	 * @brief RtcNaming 饹ǥȥ饯
	 *
	 * @else
	 *
	 * @brief A destructor of RtcNaming class.
	 *
	 * @endif
	 */
	~RtcNaming();
	
	
	/*!
	 * @if jp
	 * @brief RtcNaming 饹ǥȥ饯
	 *
	 * @param orb ORB ؤΥݥ
	 *
	 * @else
	 *
	 * @brief Initialize RtcNaming object
	 *
	 * @param orb pointer tod ORB
	 *
	 * @endif
	 */
    bool initNaming(const CORBA::ORB_ptr orb);
	
	
    //============================================================
    // << Create and Bind Methods >>
    //============================================================
	
	
	/*!
	 * @if jp
	 * @brief ͡ߥ󥰥ƥȤ
	 *
	 * Ϳ줿͡ߥ󥰥ƥȾ id,kind ΥƥȤ
	 * 
     * @param context ƥȤƥ͡ߥ󥰥ƥ
     * @param id ƥ ID
     * @param kind ƥ KIND
	 *
	 * @else
	 *
	 * @brief Create naming context
	 *
     * Create a naming context using Context_var, id and kind
	 *
     * @param context Naming context to be created new context
     * @param id context ID
     * @param kind context KIND
	 *
	 * @endif
	 */
    CosNaming::NamingContextExt_var
    createContext(CosNaming::NamingContextExt_var context,
				  const std::string& id, const std::string& kind);
	
	
	/*!
	 * @if jp
	 * @brief ۥȥƥȤ
	 *
	 * ۥȥƥȥ٥˥ۥ̾ΥƥȤ
	 * 
     * @param id ƥ̾ȤͿۥ̾
	 *
	 * @else
	 *
	 * @brief Create host context
	 *
     * Create host context on the default naming service.
	 *
     * @param id host name as a context id 
	 *
	 * @endif
	 */
    bool createHostContext(const std::string& id);
	
	
	/*!
	 * @if jp
	 * @brief ޥ͡㥳ƥȤ
	 *
	 * ޥ͡㥳ƥȥ٥˥ޥ̾͡ΥƥȤ
	 * 
     * @param id ƥ̾ȤͿޥ̾͡
	 *
	 * @else
	 *
	 * @brief Create manager context
	 *
     * Create manager context on the default naming service.
	 *
     * @param id manager name as a context id 
	 *
	 * @endif
	 */
    bool createManagerContext(const std::string& id);
	
	
	/*!
	 * @if jp
	 * @brief ƥꥳƥȤ
	 *
	 * ƥꥳƥȥ٥˥ޥ̾͡ΥƥȤ
	 * 
     * @param id ƥ̾ȤͿ륫ƥ̾
	 *
	 * @else
	 *
	 * @brief Create category context
	 *
     * Create category context on the default naming service.
	 *
     * @param id category name as a context id 
	 *
	 * @endif
	 */
    bool createCategoryContext(const std::string& category);
	
	
	/*!
	 * @if jp
	 * @brief ⥸塼륳ƥȤ
	 *
	 * ⥸塼륳ƥȥ٥˥ޥ̾͡ΥƥȤ
	 * 
     * @param module ƥ̾ȤͿ⥸塼̾
     * @param category ⥸塼륳ƥȤ륫ƥ̾
	 *
	 * @else
	 *
	 * @brief Create module context
	 *
     * Create module context on the default naming service.
	 *
     * @param module module name as a context id 
     * @param category category name on which module context is created
	 *
	 * @endif
	 */
    bool createModuleContext(const std::string& module,
							 const std::string& category);
	
	
	/*!
	 * @if jp
	 * @brief ֥ȤΥХ
	 *
	 * ֥ȤꤷƥȲ˥Х
	 * 
     * @param context ֥ȤХɤ륳ƥ
     * @param id ֥ ID
     * @param kind ֥ KIND
     * @param obj Хɤ륪֥ȤΥ֥ȥե
	 *
	 * @else
	 *
	 * @brief Bind object to a context
	 *
     * Bind object to specified naming context
	 *
     * @param context A context to be bound the object
     * @param id object ID
     * @param kind object KIND
     * @param obj Object pointer to be bound
	 *
	 * @endif
	 */
    bool bindObject(CosNaming::NamingContextExt_var context,
					const std::string& id, const std::string& kind,
					CORBA::Object_ptr obj);

	/*!
	 * @if jp
	 * @brief ֥ȤեѥǥХ
	 *
	 * ֥Ȥեѥ˽äƥХɤ롣
	 * 
     * @param path ֥ȤХɤ͡ߥ󥰥ĥ꡼Υեѥ
     * @param obj Хɤ륪֥ȤΥ֥ȥե
	 *
	 * @else
	 *
	 * @brief Bind object to a context specified full path string
	 *
     * Bind object to specified full path string.
	 *
     * @param path A full path string to be bound a object.
     * @param obj Object reference to be bound
	 *
	 * @endif
	 */
	bool bindObjectByFullPath(const std::string& path,
							  CORBA::Object_ptr obj);
	
	/*!
	 * @if jp
	 * @brief ݡͥȤΥХ
	 *
	 * ֥ȤꤷƥȲ˥Х
	 * 
     * @param id ݡͥȤ ID
     * @param category ݡͥȤΥƥ
     * @param obj ݡͥȤΥ֥ȥե
	 *
	 * @else
	 *
	 * @brief Bind component to a context
	 *
     * Bind object to specified naming context
	 *
     * @param id object ID
     * @param category A category bind to the obobject
     * @param obj Component object pointer to be bound
	 *
	 * @endif
	 */
    bool bindComponent(const std::string& component, const std::string& module,
					   const std::string& category, CORBA::Object_ptr obj);
	
	/*!
	 * @if jp
	 * @brief ޥ͡ΥХ
	 *
	 * ޥ͡ꤷƥȲ˥Х
	 * 
     * @param id ޥ̾͡
     * @param obj ޥ͡Υ֥ȥե
	 *
	 * @else
	 *
	 * @brief Bind manager to a context
	 *
     * Bind manager object to specified naming context
	 *
     * @param id Manager's name as a object id
     * @param obj Manager's object pointer
	 *
	 * @endif
	 */
    bool bindManager(const std::string& id, CORBA::Object_ptr obj);
	
	
    //============================================================
    // << Destroy and Unbind methods >>
    //============================================================
	
	
	/*!
	 * @if jp
	 * @brief ʬΥۥȥƥȤκ
	 *
	 * @else
	 * @brief Destroy host context
	 *
	 * @endif
	 */
    bool destroyHostContext();
	
	/*!
	 * @if jp
	 * @brief ʬΥޥ͡㥳ƥȤκ
	 *
	 * @else
	 * @brief Destroy manager context
	 *
	 * @endif
	 */
    bool destroyManagerContext();
	
	/*!
	 * @if jp
	 * @brief ƥꥳƥȤκ
	 *
	 * @param id ƥ̾
	 *
	 * @else
	 * @brief Destroy manager context
	 *
	 * @param id Category name
	 *
	 * @endif
	 */
    bool destroyCategoryContext(const std::string& id);
	
	
	/*!
	 * @if jp
	 * @brief ⥸塼륳ƥȤκ
	 *
	 * @param module ⥸塼̾
	 * @param category ƥ̾
	 *
	 * @else
	 * @brief Destroy manager context
	 *
	 * @param module Module name
	 * @param category Category name
	 *
	 * @endif
	 */
    bool destroyModuleContext(const std::string& module,
							  const std::string& category);
	
	
	/*!
	 * @if jp
	 * @brief ֥ȤΥХ
	 *
	 * ꤷƥȲΥ֥Ȥ򥢥Х
	 * 
     * @param context ֥ȤХɤ륳ƥ
     * @param id ֥ ID
     * @param kind ֥ KIND
	 *
	 * @else
	 *
	 * @brief Unbind object to a context
	 *
     * Unbind object to specified naming context
	 *
     * @param context A context to be unbound the object
     * @param id object ID
     * @param kind object KIND
	 *
	 * @endif
	 */
    bool unbindObject(CosNaming::NamingContextExt_var context,
					  const std::string& id, const std::string& kind);



	/*!
	 * @if jp
	 * @brief ֥ȤեѥǥХ
	 *
	 * ֥Ȥեѥ˽äƥХɤ롣
	 * 
     * @param path Хɤ륪֥ȤΥեѥ
	 *
	 * @else
	 *
	 * @brief Unbind object to a context specified full path string
	 *
     * Unbind object to specified full path string.
	 *
     * @param path A full path string to be bound a object.
	 *
	 * @endif
	 */
	bool unbindObjectByFullPath(const std::string& path);


	/*!
	 * @if jp
	 * @brief ݡͥȥ֥Ȥ򥢥Х
	 *
	 * RtcNaming饹ǥХɤ줿󥰥͡Υ֥Ȥ
	 * Хɤ롣
	 * 
     * @param category Хɤ륪֥ȤΥƥ̾
     * @param module Хɤ륪֥ȤΥ⥸塼̾
     * @param instance Хɤ륪֥ȤΥ󥹥̾
	 *
	 * @else
	 *
	 * @brief Unbind component object
	 *
     * Unbind object which was bound this RtcNaming object.
	 *
     * @param category Category name of the object to be unbound.
     * @param module Module name of the object to be unbound.
     * @param instance Instance name of the object to be unbound.
	 *
	 * @endif
	 */
	bool unbindLocalComponent(const std::string category,
							  const std::string module,
							  const std::string instance);
	


    //============================================================
    // << Find methods >>
    //============================================================
	
	/*!
	 * @if jp
	 * @brief ۥȥƥȤθ
	 *
	 * @param id_seq ID
	 * @param context ƥ ()
	 *
	 * @else
	 * @brief Find host context
	 *
	 * @param id_seq ID sequence to be find
	 * @param context context sequence (return value)	
	 *
	 * @endif
	 */
    bool findHostContext(const std::string& id_seq, ContextList& context);
	
	
	/*!
	 * @if jp
	 * @brief ƥꥳƥȤθ
	 *
	 * @param id_seq ID
	 * @param context ƥ ()
	 *
	 * @else
	 * @brief Find category context
	 *
	 * @param id_seq ID sequence to be find
	 * @param context context sequence (return value)	
	 *
	 * @endif
	 */
    bool findCategoryContext(const std::string& id_seq, ContextList& context);
	
	
	/*!
	 * @if jp
	 * @brief ⥸塼륳ƥȤθ
	 *
	 * @param id_seq ID
	 * @param context ƥ ()
	 *
	 * @else
	 * @brief Find module context
	 *
	 * @param id_seq ID sequence to be find
	 * @param context context sequence (return value)	
	 *
	 * @endif
	 */
    bool findModuleContext(const std::string& id_seq, ContextList& context);
	
	
	/*!
	 * @if jp
	 * @brief ޥ͡㥳ƥȤθ
	 *
	 * @param id_seq ID
	 * @param context ƥ ()
	 *
	 * @else
	 * @brief Find manager context
	 *
	 * @param id_seq ID sequence to be find
	 * @param context context sequence (return value)	
	 *
	 * @endif
	 */
    bool findManagerContext(const std::string& id_seq, ContextList& context);
	
	
	/*!
	 * @if jp
	 * @brief ޥ͡㥪֥ȥե󥹤θ
	 *
	 * @param id_seq ID
	 * @param object ֥ȥե ()
	 *
	 * @else
	 * @brief Find and get manager object reference
	 *
	 * @param id_seq ID sequence to be find
	 * @param object objecte reference list (return value)	
	 *
	 * @endif
	 */
    bool findManager(std::string& id, ObjectList& objects);
	
	
	/*!
	 * @if jp
	 * @brief ݡͥȥ֥ȥե󥹤θ
	 *
	 * @param id_seq ID
	 * @param object ֥ȥե ()
	 *
	 * @else
	 * @brief Find and get component object reference
	 *
	 * @param id_seq ID sequence to be find
	 * @param object objecte reference list (return value)	
	 *
	 * @endif
	 */
    bool findComponents(const std::string& id_seq, ObjectList& objects);
	
	
  protected:
	/*!
	 * @if jp
	 * @brief ͡ߥ󥰥ƥȤκƵŪ
	 *
	 * @param context 򳫻Ϥ륳ƥ
	 *
	 * @else
	 * @brief Destroy naming context recursive
	 *
	 * @param context Starting point naming context to be deleted.
	 *
	 * @endif
	 */
    bool destroyRecursive(CosNaming::NamingContextExt_var context);
	
	
	/*!
	 * @if jp
	 * @brief ֥ȥե󥹤ƵŪ˼
	 *
	 * @param context 򳫻Ϥ륳ƥ
	 * @param path ѥ
	 * @param obj ֥ȥե ()
	 *
	 * @else
	 * @brief Find object reference recursive
	 *
	 * @param context Starting point naming context 
	 * @param path Search path name
	 * @param obj Object reference list (return value)
	 *
	 * @endif
	 */
    void findObjectsRecursive(CosNaming::NamingContextExt_ptr context,
							  const std::string& path, ObjectList& obj);
	
	/*!
	 * @if jp
	 * @brief ͡ߥ󥰥ƥȤƵŪ˼
	 *
	 * @param context 򳫻Ϥ륳ƥ
	 * @param path ѥ
	 * @param context_list ƥ ()
	 *
	 * @else
	 * @brief Find naming context recursive
	 *
	 * @param context Starting point naming context 
	 * @param path Search path name
	 * @param context_list Naming context list (return value)
	 *
	 * @endif
	 */
    void findContextRecursive(CosNaming::NamingContextExt_var context,
							  const std::string& path, ContextList& context_list);
	
	
	/*!
	 * @if jp
	 * @brief ƥȡ֥ȤƵŪ˥Х
	 *
	 * @param context Хɤ򳫻Ϥ륳ƥ
	 * @param path ѥ
	 * @param obj ֥ȥե
	 *
	 * @else
	 * @brief Bind context/object recursive
	 *
	 * @param context Starting point naming context 
	 * @param path Binding path name
	 * @param obj Object reference to be bound
	 *
	 * @endif
	 */
	void bindObjectRecursive(CosNaming::NamingContextExt_var context,
							 const std::string& path,
							 CORBA::Object_ptr obj);
	
	
  private:
	/*!
	 * @if jp
	 * @brief ʸʬ
	 * @else
	 * @brief Strig split function
	 * @endif
	 */
    int split(const std::string& input, const std::string& delimiter,
			  std::vector<std::string>& results);
	
	/*!
	 * @if jp
	 * @brief Ϳ줿ʸեѥˤ
	 * @else
	 * @brief Convert given string to full path string
	 *
	 * If given path is "hoge/munya" and level is 5, it is converted to 
	 * "*./*./*./hoge/munya".
	 *
	 * @param path path which have to converted to full path
	 * @param level compensation level
	 *
	 * @endif
	 */
    std::string makeFullPath(std::string path, int level);
	
	/*!
	 * @if jp
	 * @brief ߤΥΡɤڤФ
	 *
	 * @param path ѥ
	 * @param node ڤФ줿Ρ̾ ()
	 * @param remain ĤΥѥ ()
	 *
	 * @else
	 * @brief Convert given string to full path string
	 *
	 * @param path Given path
	 * @param node Picked up path node name (return value)
	 * @param remain Remain path string (return value)
	 *
	 * @endif
	 */
    bool pickupNode(const std::string& path, std::string& node, std::string& remain);
	
	/*!
	 * @if jp
	 * @brief Ϳ줿ʸ ID  KIND ʬ䤹
	 * Kind ξ ".*" 롣
	 *
	 * @param node Ρ̾
	 * @param id ID ()
	 * @param kind KIND ()
	 *
	 * @else
	 * @brief Convert given string to full path string
	 *
	 * @param node Node name
	 * @param id ID (return value)
	 * @param kind KIND (return value)
	 *
	 * @endif
	 */
    void nodeToIdKindAny(const std::string& node,
					  std::string& id, std::string& kind);

	/*!
	 * @if jp
	 * @brief Ϳ줿ʸ ID  KIND ʬ䤹
	 *
	 * @param node Ρ̾
	 * @param id ID ()
	 * @param kind KIND ()
	 *
	 * @else
	 * @brief Convert given string to full path string
	 *
	 * @param node Node name
	 * @param id ID (return value)
	 * @param kind KIND (return value)
	 *
	 * @endif
	 */
    void nodeToIdKind(const std::string& node,
					  std::string& id, std::string& kind);
	
  private:
	
	/*!
	 * @if jp
	 * @brief ͡ߥ󥰥ӥΥ֥ȥե
	 * @else
	 * @brief Object reference of Naming Service
	 * @endif
	 */
    CORBA::Object_var m_nameRef;
	
	/*!
	 * @if jp
	 * @brief 롼ȥƥ
	 * @else
	 * @brief Root naming context
	 * @endif
	 */
    CosNaming::NamingContextExt_var m_rootContext;
	
	/*!
	 * @if jp
	 * @brief ۥȥƥ
	 * @else
	 * @brief Host naming context
	 * @endif
	 */
    CosNaming::NamingContextExt_var m_hostContext;
	std::string m_hostName;

	/*!
	 * @if jp
	 * @brief ޥ͡㥳ƥ
	 * @else
	 * @brief Manager naming context
	 * @endif
	 */
    CosNaming::NamingContextExt_var m_mngrContext;
	std::string m_mngrName;

	/*!
	 * @if jp
	 * @brief ƥꥳƥȥޥå
	 * @else
	 * @brief Category naming context map
	 * @endif
	 */
    std::map<std::string, CosNaming::NamingContextExt_var> m_cateContext;
	
	/*!
	 * @if jp
	 * @brief ݡͥȥƥȥޥå
	 * @else
	 * @brief Category naming context map
	 * @endif
	 */
    std::map<std::string, std::map<std::string, CosNaming::NamingContextExt_var> > m_compContext;
	
	/*!
	 * @if jp
	 * @brief ͡ߥ󥰥ƥȤκ
	 * @else
	 * @brief maximum number of naming context to get at once
	 * @endif
	 */
    int m_bindListNum;
	
	/*!
	 * @if jp
	 * @brief ֥ȥꥹ
	 * @else
	 * @brief Object reference list
	 * @endif
	 */
    std::vector<CORBA::Object_ptr> m_vObjects;
	
  };
  
}; // end of namespace RTM

#endif // end of __RtcNaming_h__
