// -*- C++ -*-
/*!
 * @file RtcPortBase.h
 * @brief RT component base class
 * @date $Date: 2005/05/16 05:50:33 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

/*
 * $Log$
 *
 *
 */

#ifndef RtcPortBase_h
#define RtcPortBase_h

#include <list>
#include <ace/Task.h>

#include "rtm/idl/RTCPortBase.h"

namespace RTM
{
  class RtcPortBase
  {
  public:
	RtcPortBase(){};

	void append(Subscription& subs)
	{
	  ACE_Guard<ACE_Thread_Mutex> guard(m_Subscriptions.m_Mutex);
	  m_Subscriptions.m_List.push_back(subs);
	}
	
	void append(Subscription& subs)
	{
	  ACE_Guard<ACE_Thread_Mutex> guard(m_Subscriptions.m_Mutex);
	  m_Subscriptions.m_List.push_back(subs);
	}
	
	Subscription& find(char* id)
	  throw (NoSubscription)
	{
	  ACE_Guard<ACE_Thread_Mutex> guard(m_Subscriptions.m_Mutex);

	  Subs_it it;
	  it = find_if(m_Subscriptions.m_List.begin(),
				   m_Subscriptions.m_List.end(),
				   eq_name(id));
	  if (it == m_Subscriptions.m_List.end())
		{
		  throw NoSubscription(id);
		}
	  return *it;
	};
	
  protected:
	class eq_id
	{
	public:
	  const string m_id;
	  eq_name(const char* id) : m_id(id){};
	  bool operator()(Subscription subs)
	  {
		string id(subs.id);
		return m_id == id;
	  };
	};

	struct Subscriptions
	{
	  ACE_Thread_Mutex m_Mutex;
	  std::list<Subscription> m_List;
	};
	Subscriptions m_Subscriptions;
	typedef list<Subscription>::iterator Subs_it;
  };

};

#endif // RtcPortBase_h
